function fs = choose_manifold(manifold, retraction, vector_transport)
% This function return some necessary function handles of some manifolds which are required in RBroydenFamily code.
% 
% INPUT:
% Some examples of manifold. For each manifold, when specify a pair of retraction and vector transport, the required objects are given.
% See sphere.m, stiefel.m orthogroup.m and grassmann.m for details about objects 
% manifold : the manifold which objective function is on.
%         '1', sphere : S^{n - 1}
%         '2', Stiefel manifold : St(p, n)
%         '3', the orthogonal group : O(n)
%         '4', the Grassmann manifold : Gr(p, n)
% retraction : a number represents a retraction. See sphere.m, stiefel.m orthogroup.m and grassmann.m for details
% vector transport : a number represents a vector transport. See sphere.m, stiefel.m orthogroup.m and grassmann.m for details
% OUTPUT:
% Output function handles about the given manifold.
% See sphere.m, stiefel.m orthogroup.m and grassmann.m for details
%
% By Wen Huang

    if(manifold == 1)
        fs = sphere(retraction, vector_transport);
    end

    if(manifold == 2)
        fs = stiefel(retraction, vector_transport);
    end
    
    if(manifold == 3)
        fs = orthogroup(retraction, vector_transport);
    end
    
    if(manifold == 4)
        fs = grassmann(retraction, vector_transport);
    end
end
