function [X, F, G, time, nf, ng, T, nR] = driver_SU_linesearch(A, Q, fns, params)
  
    k_max = params.k_max;
    fns.f = @(x)f(x, A, Q);
    fns.gf = @(x)gf(x, A, Q, k_max);
    fns.R = @(x, xi)R_svd(x, xi, k_max);
    fns.beta = @beta;
    fns.inpro = @(x, v1, v2)innerproduct(x, v1, v2);
    fns.STV = @STV;
    fns.TVaddTV = @TVaddTV;
    fns.IsStopped = @(X, F, G, T, params, fns)IsStopped(X, F, G, T, params, fns);
    
    [X, F, G, time, nf, ng, T, nR] = RSD(fns,params);
   
end

function [output, params] = IsStopped(X, F, G, T, params, fns)
    x = X{end};
    if (G(end) < params.tol) %(G(end) / params.Gfull0 < params.tol) 
        output = 1;
    else
        output = 0;
    end
    
end

function [output, x] = f(x, A, Q)
    [m, n] = size(A);
    x.s = x.u * x.d * x.v';
    switch size(Q, 2)
        case m
            temp = A - x.s;
            output = 0;
            x.QX = zeros(m, n);
            for i = 1 : n
                Qi = Q(:, :, i);
                x.QX(:, i) = Qi * temp(:, i);
                output = output + temp(:, i)' * x.QX(:, i);
            end  
        case m * n
            temp = reshape(A - x.s, [], 1);
            x.QX = Q * temp;
            output = temp' * x.QX;
    end
end

function [output, x] = gf(x, A, Q, k_max) % gradient of f
    [m, n] = size(A);
    
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = x.u * x.d * x.v';  
        end
        switch size(Q, 2)
            case m
                temp = A - x.s;
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:, i) = Qi * temp(:, i);
                end
            case m * n
                x.QX = Q * reshape(A - x.s, [], 1);
        end
    end
    x.grad.TV = reshape(- 2 * x.QX, m, n);
    output = project(x, x.grad, k_max);
end

function output = project(x, xi, k_max) % project a vector to tangent space of s
    s = size(x.d, 1);
    if (s < k_max)
        temp1 = x.u' * xi.TV;
        gradfr = x.u * (temp1 - ((temp1 * x.v) * x.v')) + (xi.TV * x.v) * x.v';
        temp = xi.TV - gradfr;
        [U, D, V] = svds(temp, k_max - s);
        output.TV = gradfr + U * D * V';
    else
        temp1 = x.u' * xi.TV;
        output.TV = x.u * (temp1 - ((temp1 * x.v) * x.v')) + (xi.TV * x.v) * x.v';
    end
        
end

function output = innerproduct(x, v1, v2) % inner product
    output = sum(sum(v1.TV .* v2.TV));
end

function output = R_svd(x, xi, k_max) % retraction by SVD
    if(~isfield(x, 's'))
       	x.s = x.u * x.d * x.v';  
    end  
    [U, D, V] = svd(x.s + xi.TV);
    output.u = U(:, 1:k_max);
    output.d = sparse(D(1:k_max, 1:k_max));
    output.v = V(:, 1:k_max);
end

function output = beta(x1, eta, x2)
    output = 1;
end

function output = STV(x, a, eta)
    output.TV = a * eta.TV;
end

function output = TVaddTV(x, eta1, eta2)
    output.TV = eta1.TV + eta2.TV;
end


