function [X, F, G, times, T] = driver_wlr(A, Q, fns, params)

	fns.IsStopped = @(X, F, G, T, params, fns)IsStopped(X, F, G, T, params, fns);

    fns.f = @(x)f(x, A, Q);
    fns.gf = @(x)gf(x, A, Q);
    fns.gf_full = @(x)full_gf(x, A, Q);

    fns.inpro = @(x, v1, v2)innerproduct(x, v1, v2);
    fns.g = @(x, v1, v2)innerproduct(x, v1, v2);
    fns.proj = @(x, xi)project(x, xi);
    
    fns.Hv = @Hv;
    fns.rank1operator = @rank1operator;
    fns.operadd = @operadd;
    fns.SRTV = @SRTV;
    fns.STV = @STV;
    fns.TVaddTV = @TVaddTV;
    
    fns.inpro_full = @(x, v1, v2)innerproduct(x, v1, v2);
    fns.increase_r = @(x) increaserank(x, A, Q, params);
    fns.reduce_rank = @(x, r)reduce_rank(x, r);
    fns.resetHB = @(x)reset_parameter(x);
  	fns.check_rank = @(x, tol)check_rank(x, tol);
    fns.full_c = @(x, xi)full_c(x, xi);
   
    
    if params.retraction == 1
        fns.R = @(x, xi)R_svd(x, xi);
        fns.R_r = @(x, xi)R_svd_r(x, xi);
        fns.beta = @beta_svd;
    elseif params.retraction == 2
        fns.R = @(x, xi)R_polar(x, xi);
        fns.R_r = @(x, xi)R_polar_r(x, xi);
        fns.beta = @beta_polar;
    elseif params.retraction == 3
        fns.R = @(x, xi)R_qf1(x, xi);
       	fns.R_r = @(x, xi)R_qf1_r(x, xi);
        fns.beta = @beta_qf;
    elseif params.retraction == 4
        fns.R = @(x, xi)R_qf2(x, xi);
        fns.R_r = @(x, xi)R_qf2_r(x, xi);
        fns.beta = @beta_qf;
    end
    
    fns.Tranv = @TranspVect;
    fns.hessianv = @(x, eta)Hess(x, eta, A, Q);

    [X, F, G, times, T] = MROM(fns,params);
end

%% stopping criterion on fixed-rank manifold
function [output, params] = IsStopped(X, F, G, T, params, fns)
    x = X{end};
    if (x.r < x.l)
        output = 1;
        params.flag = 0;
    elseif (G(end) < params.epsilon_3)
        output = 1;
        params.flag = 1;
    else
        output = 0;
    end
end

%% evaluate function values
function [output, x] = f(x, A, Q)
    [m, n] = size(A);
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = (x.u * x.d) * x.v';   % x = u * d * v^T
        end
        switch size(Q, 2)
            case n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;    % err = A - x
                end
                x.QX = Q * x.err;       % QX = Q*(A-x)
            case m
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:,i) = Qi * x.err(:, i);
                end
            case m*n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err(:);
        end
    end
    switch size(Q, 2)
        case n
            output = x.err(:)' * x.QX(:);  % (A-x)^T * Q * (A-x)
        case m
            output = 0;
            for i = 1 : n
                output = output + x.err(:, i)' * x.QX(:, i);
            end
        case m*n
            output = x.err(:)' * x.QX;
    end
end

%% local gradient of f 
function [output, x] = gf(x, A, Q) 
   [m, n] = size(A);
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = x.u * x.d * x.v';
        end
        switch size(Q, 2)
            case n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err;   
            case m
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:,i) = Qi * x.err(:, i);
                end
            case m*n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err(:);
        end
    end
    x.gradF.TV = reshape(-2 * x.QX, m, n);
    output = project(x, x.gradF);   
    x.gradr = output;
end

%% project a vector to tangent space of x
function output = project(x, xi) 
    if (~isfield(xi, 'TV'))
        temp_u = xi.U + x.u * xi.D + (x.u * xi.V') * x.v; % xi * v
        temp_v = x.v * xi.U' * x.u + x.v * xi.D' + xi.V;  % xi^T * u
        output.D = x.u' * temp_u;                 % derivative of d
        output.U = temp_u - x.u * output.D;       % derivative of u
        output.V = temp_v - x.v * output.D';      % derivative of v
%         xi.TV = xi.U * x.v' + x.u*xi.D*x.v' + x.u*xi.V';
    else
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        output.D = x.u' * temp_u;
        output.U = temp_u - x.u * output.D;
        output.V = temp_v - x.v * output.D'; 
    end
end

%% full gradient of f
function [output, x] = full_gf(x, A, Q) 
    [m, n] = size(A);
    if(~isfield(x, 'gradF.TV'))
        if(~isfield(x, 'QX'))
            if(~isfield(x, 's'))
                x.s = x.u * x.d * x.v';
            end
            switch size(Q, 2)
            	case n
                    if(~isfield(x, 'err'))
                        x.err = A - x.s;
                    end
                    x.QX = Q * x.err;
                case m
                    if(~isfield(x, 'err'))
                        x.err = A - x.s;
                    end
                    x.QX = zeros(m, n);
                    for i = 1 : n
                        Qi = Q(:, :, i);
                        x.QX(:,i) = Qi * x.err(:, i);
                    end
                case m*n
                    if(~isfield(x, 'err'))
                        x.err = A - x.s;
                    end
                    x.QX = Q * x.err(:);
            end
        end
        x.gradF.TV = reshape(-2 * x.QX, m, n);
    end
	output.TV = x.gradF.TV;
    x.gradfull = output;
end

%% inner product
function output = innerproduct(x, v1, v2) 
    if(~isfield(v1, 'TV') && ~isfield(v2, 'TV'))
        output = v1.D(:)' * v2.D(:) + v1.U(:)' * v2.U(:) + v1.V(:)' * v2.V(:);
    elseif (~isfield(v2, 'TV'))
        a1 = v1.TV * x.v;
        a2 = v1.TV * v2.V;
        output = sum(sum(a1 .* (v2.U + (x.u * v2.D)) + a2 .* x.u)); 
    else
        output = sum(sum(v1.TV .* v2.TV));
    end
end

%% retraction on fixed-rank manifold by SVD
function output = R_svd(x, xi) 
    k = length(x.d);
    % compute the derivative of u, d, v seperately
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;   % xi * v
        temp_v = xi.TV' * x.u;  % xi^T * u
        xi.D = x.u' * temp_u;   % derivative of d 
        xi.U = temp_u - x.u * xi.D; % derivative of u
        xi.V = temp_v - x.v * xi.D'; % derivative of v
    end
    [Qu, Ru] = qr(xi.U, 0);
    [Qv, Rv] = qr(xi.V, 0);
    S = [x.d + xi.D, Rv'; Ru, zeros(k)];
    [Us, Ds, Vs] = svd(S);
    D = Ds(1:k, 1:k);
    % retraction of u, d, v seperately
    output.u = [x.u, Qu] * Us(:, 1:k);
    output.d = sparse(D);
    output.v = [x.v, Qv] * Vs(:, 1:k);
    [output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

function output = beta_svd(x1, eta, x2)
    output = 1; %eta.beta;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

%% retraction on fixed-rank manifold by Polar Decomposition
function output = R_polar(x, xi) 
	% compute the derivative of u, d, v seperately
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end
    Unew = x.u + xi.U / x.d;
    Vnew = x.v + xi.V / x.d';
    U = Unew / sqrtm(Unew'*Unew);
    V = Vnew / sqrtm(Vnew'*Vnew);
    D = x.d + xi.D;   
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
    [output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

function output = beta_polar(x1, eta, x2)
    output = 1; %eta.beta;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

%% retraction on fixed-rank manifold by QR (derivative of d is a full matrix)
function output = R_qf1(x, xi)
    % compute the derivative of u, d, v seperately
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
    xi.U = xi.U / x.d;
    xi.V = xi.V / x.d;
    U = qf_thin(x.u + xi.U);
    V = qf_thin(x.v + xi.V);
    D = x.d + xi.D;
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
    [output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

function output = beta_qf(x1, eta, x2)
    output = 1; %eta.v;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

%% retraction on fixed-rank manifold by QR (derivative of d is a diagonal matrix)
function output = R_qf2(x, xi) 
    k = length(x.d); 
    vd = diag(x.d);   % diagonal of D        
    vd_inv = 1./vd;   % diagonal of D^{-1}
    temp1 = vd_inv * vd' - vd * vd_inv'; % V_{D^{-1}}*V_D^{-1}'- V_D*V_{D^{-1}}'
    % compute the derivative of u, d, v seperately       
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
    % derivative of D (diagonal)
    D_d = diag(diag(xi.D));
    Z = xi.D - D_d;
    
    % temp2 = d \ temp; % inv(D)*temp
    % temp3 = temp / d; % temp*inv(D)
    temp2 = zeros(size(Z));
    temp3 = zeros(size(Z));
    for i = 1 : k
        temp2(i, :) = Z(i, :) / vd(i);
        temp3(:, i) = Z(:, i) / vd(i);
    end
    
    omega_u = (temp2 + temp2') ./ temp1;
    omega_v = (temp3 + temp3') ./ temp1;
    omega_u(1:k+1 : k*k) = 0;
    omega_v(1:k+1 : k*k) = 0;
    
    %U_d = u * omega_u + (I-uu')*xi*v*d^{-1}
    %V_d = v * omega_v + (I-vv')*xi'*u*d^{-1}
    U_d = x.u * omega_u + xi.U / x.d;
    V_d = x.v * omega_v + xi.V / x.d;
%     temp_u = (xi.v * x.v) / x.d;
%     temp_v = (xi.v' * x.u) / x.d;
%     U_d = x.u * (omega_u - (x.u' * temp_u)) + temp_u;
%     V_d = x.v * (omega_v - (x.v' * temp_v)) + temp_v; 
   
    U = qf_thin(x.u + U_d);
    V = qf_thin(x.v + V_d);
    D = x.d + D_d;
    
    DD = diag(D);
    [~, P] = sort(abs(DD), 'descend');
    I = eye(length(DD), length(DD));
    P1 = I(P, :);
    Sg = sign(DD);
    Sg(Sg == 0) = 1;
    U = U * diag(Sg) * P1';
    D = P1 * diag(Sg) * D * P1';
    V = V * P1';
    % retraction of u, d, v separately (d is a diagonal matrix)
    output.u = U(:, 1:k);
    output.d = sparse(D(1:k, 1:k));
    output.v = V(:, 1:k);
    [output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

%% retraction on fixed-rank manifold by RRR (derivative of d is a full matrix)
function output = R_RRR(x, xi)
    % compute the derivative of u, d, v seperately
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end
    [U, Ru] = qr(x.u * x.d + xi.U, 0);
    [V, Rv] = qr(x.v * x.d' + xi.V, 0);
    D = (Ru/x.d) * (x.d + xi.D) * (x.d\Rv');
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
    [output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

function output = beta_RRR(x1, eta, x2)
    output = 1; %eta.v;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

%% hessian of f on fixed-rank manifold
function output = Hess(x, eta, A, Q)
    [m, n] = size(A);
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = x.u * x.d * x.v';
        end
        switch size(Q, 2)
            case n
                if(~isfield(x, 'err'))
                	x.err = A - x.s;
                end
             	x.QX = Q * x.err;
            case m
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:,i) = Qi * x.err(:, i);
                end
            case m*n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err(:);
        end
    end
    M_s = reshape(x.QX, m, n);  % Q * (A - x)
    
    if(~isfield(eta, 'TV'))
        eta.TV = eta.U * x.v' + x.u*eta.D*x.v' + x.u*eta.V';
    end
    
    % calculate Q * eta
    switch size(Q, 2)
        case n
            M_v = Q * eta.TV;
        case m
            M_v = zeros(m, n);
            for i = 1 : n
                Qi = Q(:, :, i);
                M_v(:, i) = Qi * eta.TV(:, i);
            end
        case m*n
            eta_v = reshape(eta.TV, m*n, 1);
            M_v = reshape(Q * eta_v, m, n);
    end
   
    % derivative of (u * u^T)
    temp_u = (eta.TV * x.v) / x.d;
    temp_u = temp_u * x.u' - x.u * (x.u' * temp_u) * x.u';
    temp_u = temp_u + temp_u'; % temp_u = (u * u^T)'

    % derivative of (v * v^T)
    temp_v = x.d \ (x.u' * eta.TV);
    temp_v = x.v * temp_v - x.v * (temp_v * x.v) * x.v';
    temp_v = temp_v + temp_v'; % temp_v = (v * v^T)'
    
    temp_1 = temp_u * ((M_s * x.v) * x.v' - M_s); %(u*u^T)'*Q*(A-x)*v*v^T - (u*u^T)'*Q*(A-x)
    temp_2 = (x.u * (x.u' * M_s) - M_s) * temp_v; %u*u^T*Q*(A-x)*(v*v^T)'-u*Q*(A-x)*(v*v^T)'
    tempa = x.u * (x.u' * M_v); % u*u^T*Q*(A-x)
    temp_3 = (tempa * x.v) * x.v' - tempa; % u*u^T*Q*(A-x)*v*v^T - u*u^T*Q*(A-x)
    temp_4 = (M_v * x.v) * x.v'; % Q*eta*v*v^T
    output.TV = 2 * (temp_1 + temp_2 - temp_3 + temp_4);      
    output = project(x, output);      
end

%% increase rank
function [output, eta] = increaserank(x, A, Q, params)
    [m, n] = size(A);
    k = length(x.d);
    % local gradient
    if(~isfield(x.gradr, 'TV'))
        gradf = x.gradr.U * x.v' + x.u * x.gradr.D * x.v' + x.u * x.gradr.V';  
    else
        switch size(x.gradr.TV, 1)
            case m
                gradf = x.gradr.TV;
            case (m + n - k) * k
                temp = full_c(x, x.gradr);
                gradf = temp.TV;
        end
    end
	gradF = x.gradF.TV; % full gradient
	temp = gradF - gradf; % difference between full and local gradient
    l = x.r;
    % increase the rank
    while(norm(temp, 'fro') > params.epsilon_4 * norm(gradf, 'fro') && l < params.k_max)
        [Ut, Dt, Vt] = RandomSubspaceIter(temp, 1, 2);
        gradf = gradf + Ut * Dt * Vt';
       	temp = x.gradF.TV - gradf;
       	x.u = [x.u, Ut];
      	x.d = blkdiag(x.d, zeros(1, 1));
      	x.v = [x.v, Vt];
       	l = l + 1;
    end
    output.u = x.u;
    output.d = x.d;
    output.v = x.v;
    output.delta = x.delta;
    eta.TV = gradf;    
end

%% rank-related retraction by SVD
function output = R_svd_r(x, xi)    
    k = length(x.d);
    % compute the derivative of u, d, v seperately    
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end  
    [Qu, Ru] = qr(xi.U, 0);
    [Qv, Rv] = qr(xi.V, 0);
    S = [x.d + xi.D, Rv'; Ru, zeros(k, k)];
    [Us, Ds, Vs] = svd(S);
    D = Ds(1:k, 1:k);
    % rank-related retraction of u, d, v 
    output.u = [x.u, Qu] * Us(:, 1:k);
    output.d = sparse(D);
    output.v = [x.v, Qv] * Vs(:, 1:k);
  	[output.r, output.l] = check_rank(output, x.delta);
   	output.delta = x.delta;
end

%% rank-related retraction by polar decomposition
function output = R_polar_r(x, xi) 

    r = nnz(x.d);
    vd = diag(x.d(1:r, 1:r));
    % U_dot = u_p * K_U = (I-uu') * xi * v * d^{-1}
    % V_dot = v_p * K_v = (I-vv') * xi'* u * d^{-1}
    % D_dot = omega_u * d + D_d + d * omega_v'
    
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
        for i = 1 : r
            xi.U(:, i) = xi.U(:, i) / vd(i);
            xi.V(:, i) = xi.V(:, i) / vd(i);
        end
    end
    
    Unew = x.u + xi.U;
    Vnew = x.v + xi.V;
    U = Unew / sqrtm(Unew'*Unew);
    V = Vnew / sqrtm(Vnew'*Vnew);
    D = x.d + xi.D;
    
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
 	[output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta; 
end

%% rank-related retraction by qr (derivative of d is a full matrix)
function output = R_qf1_r(x, xi) 

    r = nnz(x.d);
    vd = diag(x.d(1:r, 1:r));
    % U_dot = u_p * K_U = (I-uu') * xi * v * d^{-1}
    % V_dot = v_p * K_v = (I-vv') * xi'* u * d^{-1}
    % D_dot = omega_u * d + D_d + d * omega_v'
    
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
        for i = 1 : r
            xi.U(:, i) = xi.U(:, i) / vd(i);
            xi.V(:, i) = xi.V(:, i) / vd(i);
        end
    end
   
    U = qf_thin(x.u+xi.U);
    V = qf_thin(x.v+xi.V);
    D = x.d + xi.D;
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
 	[output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

%% rank-related retraction by qr (derivative of d is a diagonal matrix)
function output = R_qf2_r(x, xi) 
    r = nnz(x.d);
    % x before increase rank
    ur = x.u(:, 1:r);
    dr = x.d(1:r, 1:r);
    vr = x.v(:, 1:r);
  
    u_delta = x.u(:, r+1:end);
    v_delta = x.v(:, r+1:end);
    vd = diag(dr);   % diagonal of D        
    vd_inv = 1./vd; % diagonal of D^{-1}
    temp1 = vd_inv * vd' - vd * vd_inv'; % V_{D^{-1}}*V_D^{-1}'- V_D*V_{D^{-1}}'
    
    
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end
    Ddot = diag(diag(xi.D));
	Z = xi.D - Ddot;
	temp11 = Z(1:r, 1:r);
	temp12 = Z(1:r, r+1:end);
	temp21 = Z(r+1:end, 1:r);
        
	temp2 = zeros(size(temp11));
	temp3 = zeros(size(temp11));
	for i = 1 : r
        temp2(i, :) = temp11(i, :) / vd(i);
        temp3(:, i) = temp11(:, i) / vd(i);
    end
	omega_u1 = (temp2 + temp2') ./ temp1;
  	omega_v1 = (temp3 + temp3') ./ temp1;
	omega_u1(1:r+1:r*r) = 0;
 	omega_v1(1:r+1:r*r) = 0;
    
    omega_u2 = temp21 / dr;
  	omega_v2 = temp12' / dr;

	temp_omegau = [ur*omega_u1+u_delta*omega_u2, -ur*omega_u2'];
	temp_omegav = [vr*omega_v1+v_delta*omega_v2, -vr*omega_v2'];
    %     omega_u = [omega_u1 -omega_u2'; omega_u2 zeros(r+1:k, r+1:k)];
    %     omega_v = [omega_v1 -omega_v2'; omega_v2 zeros(r+1:k, r+1:k)];
    
    temp_u(1:r, 1:r) = xi.U(1:r, 1:r) / dr;
    temp_u(r+1:end, 1:r) = xi.U(r+1:end, 1:r) / dr;
 	temp_v(1:r, 1:r) = xi.V(1:r, 1:r) / dr;
 	temp_v(r+1:end, 1:r) = xi.V(r+1:end, 1:r) / dr;
    
   	Udot = temp_omegau + temp_u;
   	Vdot = temp_omegav + temp_v;
      
    U = qf_thin(x.u + Udot);
    V = qf_thin(x.v + Vdot);
    D = x.d + Ddot;
    DD = diag(D);
    [~, P] = sort(abs(DD), 'descend');
    I = eye(length(DD), length(DD));
    P1 = I(P, :);
    Sg = sign(DD);
    Sg(Sg == 0) = 1;
    U = U * diag(Sg) * P1';
    D = P1 * diag(Sg) * D * P1';
    V = V * P1';
    output.u = U;
    output.d = sparse(D);
    output.v = V;
	[output.r, output.l] = check_rank(output, x.delta);
    output.delta = x.delta;
end

%% reduce the rank to r
function output = reduce_rank(x, r)
    output.u = x.u(:, 1:r);
    output.d = x.d(1:r, 1:r);
    output.v = x.v(:, 1:r);
    output.r = r;
    output.l = r;
    output.delta = x.delta;
end

%% reset the parameter 
function [H, B] = reset_parameter(x)
    m = size(x.u, 1);
    n = size(x.v, 1);
    H = eye(m * n);
    B = eye(m * n);
end

function output = STV(x, a, eta)
    if(~isfield(eta, 'TV'))
        output.U = a * eta.U;
        output.D = a * eta.D;
        output.V = a * eta.V;
    else
        output.TV = a * eta.TV;
    end
end

function output = TVaddTV(x, eta1, eta2)
    if(~isfield(eta1, 'TV') || ~isfield(eta2, 'TV'))
        output.U = eta1.U + eta2.U;
        output.D = eta1.D + eta2.D;
        output.V = eta1.V + eta2.V;
    else
        output.TV = eta1.TV + eta2.TV;
    end
end

function output = Hv(x, H, v)
    [m, n] = size(v.v);
    v.v = reshape(v.v, m*n, 1);
    output.v = reshape(H * v.v, m, n);
end

function output = rank1operator(x, v1, v2)
    [m, n] = size(v1.v);
    vv1 = reshape(v1.v, m * n, 1);
    vv2 = reshape(v2.v, m * n, 1);
    output = vv1 * vv2';
end

function output = operadd(x, H1, H2)
    output = H1 + H2;
end

function [output, d, x, y] = TranspVect(x, d, y, v)

        ip_old = innerproduct(x, v, v);

        Av = x.v' * y.v;
        Au = x.u' * y.u;
    %     Bv = v.V' * y.v;
    %     Bu = v.U' * y.u;

        M1 = Au' * v.D * Av;
    %     U1 = x.u * (v.D * Bv);
    %     V1 = x.v * (v.D' * Au);

    %     M2 = Bu' * Av;
        U2 = v.U * Av;
    %     V2 = x.v * Bu;

    %     M3 = Au' * Bv;
    %     U3 = x.u * Bv;
        V3 = v.V * Au;

    %     output.D = M1 + M2 + M3;
    %     temp1 = U1 + U2 + U3;
    %     output.U = temp1 - y.u * (y.u' * temp1);
    %     temp2 = V1 + V2 + V3;
    %     output.V = temp2 - y.v * (y.v' * temp2);
        output.D = M1;
        output.U = U2 - y.u * (y.u' * U2);
        output.V = V3 - y.v * (y.v' * V3);

    ip_new = innerproduct(x, output, output);
    output = STV(x, ip_old / ip_new, output);

end

%%=======================assistant function==================%%
function [r, l] = check_rank(x, tol)
    [md, nd] = size(x.d);
    l = min(md, nd);
    r = 1;
    for i = 2 : l
        if x.d(i,i) / x.d(1,1) > tol
            r = r + 1;
        end
    end    
end

function [output, R] = qf(B)
    k = size(B, 2);
    [output, R] = qr(B, 0);
    diagR = diag(R);
    diagR(diagR == 0) = 1;
    output = output * spdiags(sign(diagR), 0, k, k);
end

function [output, R] = qf_thin(B)
    [output, R] = qr(B, 0);
    diagR = diag(R);
    output = output(:, abs(diagR) > 1e-10);
    output = output * spdiags(sign(diagR), 0, size(output, 2), size(output, 2));
end

function [U, D, V] = RandomSubspaceIter(A, l, q)
 % q = 1 or 2
 % l = k + p, k is estimated rank, l is the oversampling parameter
 
    [m, n] = size(A);
    Omega = randn(n, l);
    Y = A * Omega;
    [Q, ~] = qr(Y, 0);
    for j = 1 : q
        Y = A' * Q;
        [Q, ~] = qr(Y, 0);
        Y = A * Q;
        [Q,~] = qr(Y, 0);
    end
    
    B = Q' * A;
    [U, D, V] = svd(B, 'econ');
    U = Q * U;
end