function [X, F, G, times] = driver_wlr_fixrank(A, Q, fns, params)

    fns.f = @(x)f(x, A, Q);
    fns.gf = @(x)gf(x, A, Q);
    fns.inpro = @(x, v1, v2)innerproduct(x, v1, v2);
    fns.g = @(x, v1, v2)innerproduct(x, v1, v2);
    fns.proj = @(x, xi)project(x, xi);
    
    fns.Hv = @Hv;
    fns.rank1operator = @rank1operator;
    fns.operadd = @operadd;
    fns.SRTV = @SRTV;
    fns.STV = @STV;
    fns.TVaddTV = @TVaddTV;
   
    
    if params.retraction == 1
        fns.R = @(x, xi)R_svd(x, xi);
        fns.beta = @beta;
    elseif params.retraction == 2
        fns.R = @(x, xi)R_polar(x, xi);
        fns.beta = @beta_polar;
    elseif params.retraction == 3
        fns.R = @(x, xi)R_qf1(x, xi);
        fns.beta = @beta_qf;
    elseif params.retraction == 4
        fns.R = @(x, xi)R_qf2(x, xi);
        fns.beta = @beta_qf;    
    end

    if (params.vector_transport == 5 || params.vector_transport == 11)
        fns.gf = @(x)gf_intr(x, A, Q);
        fns.Hv = @Hv_intr;
        fns.invBv = @invBv_intr;
        fns.proj = @(x, xi)project_intr(x, xi);
        fns.SRTV = @SRTV_intr;
        fns.rank1operator = @rank1operator_intr;
        fns.inpro = @(x, v1, v2)innerproduct_intr(x, v1, v2);
        fns.g = @(x, v1, v2)innerproduct_intr(x, v1, v2);
        if params.retraction == 1
            fns.R = @(x, xi)R_svd_intr(x, xi);
            fns.beta = @beta;
        elseif params.retraction == 2
            fns.R = @(x, xi)R_polar_intr(x, xi);
            fns.beta = @beta_polar_intr;
        elseif params.retraction == 3
            fns.R = @(x, xi)R_qf1_intr(x, xi);
            fns.beta = @beta_qf_intr;
        elseif params.retraction == 4
            fns.R = @(x, xi)R_qf2_intr(x, xi);
            fns.beta = @beta_qf_intr;    
        end
    end
    
    fns.Tranv = @TranspVect;
    fns.hessianv = @(x, eta)Hess(x, eta, A, Q);

    if params.method == 1
        [X, F, G, times] = RTR_SD(fns,params);
    elseif params.method == 2
        [X, F, G, times] = RTR_SR1(fns,params);
    elseif params.method == 3
        [X, F, G, times] = LRTR_SR1(fns,params);
    elseif params.method == 4
        [X, F, G, times] = RTR_Newton(fns, params);
    elseif params.method == 5
        [X, F, G, times] = RBFGS(fns,params);
    elseif params.method == 6
        [X, F, G, times] = LRBFGS(fns,params);
    elseif params.method == 7
        [X, F, G, times] = RSD(fns,params);
    elseif params.method == 8
        [X, F, G, times] = RCG(fns,params);
    elseif params.method == 9
        [X, F, G, times] = RNewton(fns,params);
    end

end

function [output, x] = f(x, A, Q)
    [m, n] = size(A);
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = (x.u * x.d) * x.v';
        end
        switch size(Q, 2)
            case n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err;
            case m
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:,i) = Qi * x.err(:, i);
                end
            case m*n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err(:);
        end
    end
    switch size(Q, 2)
        case n
            output = x.err(:)' * x.QX(:);
        case m
            output = 0;
            for i = 1 : n
                output = output + x.err(:, i)' * x.QX(:, i);
            end
        case m*n
            output = x.err(:)' * x.QX;
    end
end

function [output, x] = gf(x, A, Q) % gradient of f
   [m, n] = size(A);
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = x.u * x.d * x.v';
        end
        switch size(Q, 2)
            case n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err;   
            case m
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:,i) = Qi * x.err(:, i);
                end
            case m*n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err(:);
        end
    end
    x.gradF.TV = reshape(-2 * x.QX, m, n);
    output = project(x, x.gradF);   
    x.gradr = output;
end

function output = project(x, xi) % project a vector to tangent space of s
    if (~isfield(xi, 'TV'))
        xi.TV = xi.U * x.v' + x.u*xi.D*x.v' + x.u*xi.V';
    end
    temp_u = xi.TV * x.v;
    temp_v = xi.TV' * x.u;
    output.D = x.u' * temp_u;
    output.U = temp_u - x.u * output.D;
    output.V = temp_v - x.v * output.D'; 
end

function [output, x] = gf_intr(x, A, Q) % gradient of f intrinsic
    [m, k] = size(x.u);
    n = size(x.v, 1);
    if(~isfield(x, 'gradF.TV'))
        if(~isfield(x, 'QX'))
            if(~isfield(x, 's'))
                x.s = x.u * x.d * x.v';
            end
            switch size(Q, 2)
                case n
                    if(~isfield(x, 'err'))
                        x.err = A - x.s;
                    end
                    x.QX = Q * x.err; 
                case m
                    if(~isfield(x, 'err'))
                        x.err = A - x.s;
                    end
                    x.QX = zeros(m, n);
                    for i = 1 : n
                        Qi = Q(:, :, i);
                        x.QX(:,i) = Qi * x.err(:, i);
                    end
                case m*n
                    if(~isfield(x, 'err'))
                        x.err = A - x.s;
                    end
                    x.QX = Q * x.err(:);
            end
        end
        x.gradF.TV = reshape(-2 * x.QX, m, n);
    end
    if(~isfield(x, 'uperp'))
        x.uperp = spnull(x.u');  
    end
    if(~isfield(x, 'vperp'))
        x.vperp = spnull(x.v');  
    end
    
    Omega = x.u' * x.gradF.TV * x.v;
    D_omega = diag(Omega);
    K1 = x.uperp' * x.gradF.TV * x.v;
    K2 = x.u' * x.gradF.TV * x.vperp;
    
    if(~isfield(x, 'dim'))
        x.dim = (m + n - k) * k;
    end
    if(~isfield(x, 'ind_1'))
        x.ind_1 = find(tril(ones(k,k), -1));
    end
    if(~isfield(x, 'ind_2'))
        x.ind_2 = find(triu(ones(k,k), 1));
    end
    
    output.TV = zeros(x.dim, 1);
    output.TV(1 : 0.5*k*(k-1)) = Omega(x.ind_1);
    output.TV(0.5*k*(k-1)+1 : 0.5*k*(k-1)+k) = D_omega; 
    output.TV(0.5*k*(k-1)+k+1 : k*k) = Omega(x.ind_2);
    output.TV(k*k+1 : m*k) = reshape(K1, [], 1);
    output.TV(m*k+1 : x.dim) = reshape(K2, [], 1); 
    x.gradr = output;

end

% project a vector to tangent space of s with intrinsic method
function output = project_intr(x, xi) 
    output = xi;
end

function output = innerproduct(x, v1, v2) % inner product
    if(~isfield(v2, 'TV'))
        output = v1.D(:)' * v2.D(:) + v1.U(:)' * v2.U(:) + v1.V(:)' * v2.V(:);
    else
        output = sum(sum(v1.TV .* v2.TV));
    end
end

function output = innerproduct_intr(x, v1, v2) % inner product
    output = v1.TV' * v2.TV;
end

function output = R_svd(x, xi) % retraction by SVD
    k = length(x.d);
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end
    [Qu, Ru] = qr(xi.U, 0);
    [Qv, Rv] = qr(xi.V, 0);
    S = [x.d + xi.D, Rv'; Ru, zeros(k)];
    [Us, Ds, Vs] = svd(S);
    D = Ds(1:k, 1:k);
    output.u = [x.u, Qu] * Us(:, 1:k);
    output.d = sparse(D);
    output.v = [x.v, Qv] * Vs(:, 1:k);
end

function output = beta_svd(x1, eta, x2)
    output = eta.beta;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

function output = R_svd_intr(x, xi) % retraction by SVD intrinsic  
    xi = full_c(x, xi);
    k = length(x.d); %x.r;
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
   
    [Qu, Ru] = qr(xi.U, 0);
    [Qv, Rv] = qr(xi.V, 0);
    S = [x.d + xi.D, Rv'; Ru, zeros(k)];
    [Us, Ds, Vs] = svd(S);
    D = Ds(1:k, 1:k);
    output.u = [x.u, Qu] * Us(:, 1:k);
    output.d = sparse(D);
    output.v = [x.v, Qv] * Vs(:, 1:k);
    output.uperp = spnull(output.u');
    output.vperp = spnull(output.v');
    
end

function output = beta_svd_intr(x1, eta, x2)
    output = eta.beta;
%     eta = full_c(x1, eta);
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee)); 
end

function output = R_polar(x, xi) % retraction by Polar Decomposition
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end
    Unew = x.u + xi.U / x.d;
    Vnew = x.v + xi.V / x.d';
    U = Unew / sqrtm(Unew'*Unew);
    V = Vnew / sqrtm(Vnew'*Vnew);
    D = x.d + xi.D;
    
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
end

function output = beta_polar(x1, eta, x2)
    output = 1; %eta.beta;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

function output = R_polar_intr(x, xi) % retraction by polar decomposition intrinsic 
    xi = full_c(x, xi);   
    if(~all(isfield(xi, {'U', 'D', 'V'}) == 1))
        temp_u = xi.TV * x.v;
        temp_v = xi.TV' * x.u;
        xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * xi.D';
    end
    Unew = x.u + xi.U / x.d;
    Vnew = x.v + xi.V / x.d';
    U = Unew / sqrtm(Unew'*Unew);
    V = Vnew / sqrtm(Vnew'*Vnew);
    D = x.d + xi.D;
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
end

function output = beta_polar_intr(x1, eta, x2)
    output = eta.beta;
%     eta = full_c(x1, eta);
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee)); 
end

function output = R_qf1(x, xi) % retraction by QR   
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
    xi.U = xi.U / x.d;
    xi.V = xi.V / x.d;
    U = qf_thin(x.u + xi.U);
    V = qf_thin(x.v + xi.V);
    D = x.d + xi.D;
    [us, ds, vs] = svd(D);
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
end

function output = beta_qf(x1, eta, x2)
    output = 1; %eta.v;
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee));
end

function output = R_qf1_intr(x, xi) % retraction by QR intrinsic
    xi = full_c(x, xi);
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
    xi.U = xi.U / x.d;
    xi.V = xi.V / x.d;
    U = qf_thin(x.u + xi.U);
    V = qf_thin(x.v + xi.V);
    D = x.d + xi.D;  
    [us, ds, vs] = svd(D);
   
    output.u = U * us;
    output.d = sparse(ds);
    output.v = V * vs;
    output.uperp = spnull(output.u');
    output.vperp = spnull(output.v');
end

function output = beta_qf_intr(x1, eta, x2)
    output = eta.beta;
%     eta = full_c(x1, eta);
%     Tee = Tranv_R_qf(x1, eta, x2, eta);
%     output = sqrt(innerproduct(x1, eta, eta) / innerproduct(x2, Tee, Tee)); 
end

function output = R_qf2(x, xi) % retraction by QR
    k = length(x.d); 
    vd = diag(x.d);   % diagonal of D        
    vd_inv = 1./vd;   % diagonal of D^{-1}
    temp1 = vd_inv * vd' - vd * vd_inv'; % V_{D^{-1}}*V_D^{-1}'- V_D*V_{D^{-1}}'
    
    
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
    % derivative of D (diagonal)
    D_d = diag(diag(xi.D));
    Z = xi.D - D_d;
    
    % temp2 = d \ temp; % inv(D)*temp
    % temp3 = temp / d; % temp*inv(D)
    temp2 = zeros(size(Z));
    temp3 = zeros(size(Z));
    for i = 1 : k
        temp2(i, :) = Z(i, :) / vd(i);
        temp3(:, i) = Z(:, i) / vd(i);
    end
    
    omega_u = (temp2 + temp2') ./ temp1;
    omega_v = (temp3 + temp3') ./ temp1;
    omega_u(1:k+1 : k*k) = 0;
    omega_v(1:k+1 : k*k) = 0;
    
    %U_d = u * omega_u + (I-uu')*xi*v*d^{-1}
    %V_d = v * omega_v + (I-vv')*xi'*u*d^{-1}
    U_d = x.u * omega_u + xi.U / x.d;
    V_d = x.v * omega_v + xi.V / x.d;
%     temp_u = (xi.v * x.v) / x.d;
%     temp_v = (xi.v' * x.u) / x.d;
%     U_d = x.u * (omega_u - (x.u' * temp_u)) + temp_u;
%     V_d = x.v * (omega_v - (x.v' * temp_v)) + temp_v; 
   
    U = qf_thin(x.u + U_d);
    V = qf_thin(x.v + V_d);
    D = x.d + D_d;
    
    DD = diag(D);
    [~, P] = sort(abs(DD), 'descend');
    I = eye(length(DD), length(DD));
    P1 = I(P, :);
    Sg = sign(DD);
    Sg(Sg == 0) = 1;
    U = U * diag(Sg) * P1';
    D = P1 * diag(Sg) * D * P1';
    V = V * P1';
    
    output.u = U(:, 1:k);
    output.d = sparse(D(1:k, 1:k));
    output.v = V(:, 1:k);
end

function output = R_qf2_intr(x, xi) % retraction by QR intrinsic
    k = length(x.d); 
    vd = diag(x.d);   % diagonal of D        
    vd_inv = 1./vd;   % diagonal of D^{-1}
    temp1 = vd_inv * vd' - vd * vd_inv'; % V_{D^{-1}}*V_D^{-1}'- V_D*V_{D^{-1}}'
 
    xi = full_c(x, xi);
    if ~all(isfield(xi,{'U','D','V'}) == 1)
        temp_u = xi.TV * x.v; 
        temp_v = xi.TV' * x.u;
     	xi.D = x.u' * temp_u;
        xi.U = temp_u - x.u * xi.D;
        xi.V = temp_v - x.v * (x.v' * temp_v);
    end
%     temp = x.u' * xi.v * x.v;
    D_d = diag(diag(xi.D)); % derivative of D
    temp = xi.D - D_d; 
    
    % temp2 = d \ temp; % inv(D)*temp
    % temp3 = temp / d; % temp*inv(D)
    temp2 = zeros(size(temp));
    temp3 = zeros(size(temp));
    for i = 1 : k
        temp2(i, :) = temp(i, :) / vd(i);
        temp3(:, i) = temp(:, i) / vd(i);
    end
    
    omega_u = (temp2 + temp2') ./ temp1;
    omega_v = (temp3 + temp3') ./ temp1;
    omega_u(1:k+1 : k*k) = 0;
    omega_v(1:k+1 : k*k) = 0;
    
    %U_d = u * omega_u + (I-uu')*xi*v*d^{-1}
    %V_d = v * omega_v + (I-vv')*xi'*u*d^{-1}
    U_d = x.u * omega_u + xi.U / x.d;
    V_d = x.v * omega_v + xi.V / x.d;
%     temp_u = (xi.v * x.v) / x.d;
%     temp_v = (xi.v' * x.u) / x.d;
%     U_d = x.u * (omega_u - (x.u' * temp_u)) + temp_u;
%     V_d = x.v * (omega_v - (x.v' * temp_v)) + temp_v; 
   
    U = qf_thin(x.u + U_d);
    V = qf_thin(x.v + V_d);
    D = x.d + D_d;
    
    DD = diag(D);
    [~, P] = sort(abs(DD), 'descend');
    I = eye(length(DD), length(DD));
    P1 = I(P, :);
    Sg = sign(DD);
    Sg(Sg == 0) = 1;
    U = U * diag(Sg) * P1';
    D = P1 * diag(Sg) * D * P1';
    V = V * P1';
    
    output.u = U;
    output.d = sparse(D(1:k, 1:k));
    output.v = V;
    output.uperp = spnull(U');
    output.vperp = spnull(V');
end

function output = Hess(x, eta, A, Q)
    [m, n] = size(A);
    if(~isfield(x, 'QX'))
        if(~isfield(x, 's'))
            x.s = x.u * x.d * x.v';
        end
        switch size(Q, 2)
            case n
                if(~isfield(x, 'err'))
                	x.err = A - x.s;
                end
             	x.QX = Q * x.err;
            case m
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = zeros(m, n);
                for i = 1 : n
                    Qi = Q(:, :, i);
                    x.QX(:,i) = Qi * x.err(:, i);
                end
            case m*n
                if(~isfield(x, 'err'))
                    x.err = A - x.s;
                end
                x.QX = Q * x.err(:);
        end
    end
    M_s = reshape(x.QX, m, n);
    
    if(~isfield(eta, 'TV'))
        eta.TV = eta.U * x.v' + x.u*eta.D*x.v' + x.u*eta.V';
    end
    
    switch size(Q, 2)
        case n
            M_v = Q * eta.TV;
        case m
            M_v = zeros(m, n);
            for i = 1 : n
                Qi = Q(:, :, i);
                M_v(:, i) = Qi * eta.TV(:, i);
            end
        case m*n
            eta_v = reshape(eta.TV, m*n, 1);
            M_v = reshape(Q * eta_v, m, n);
    end
   
    
    temp_u = (eta.TV * x.v) / x.d;
    temp_u = temp_u * x.u' - x.u * (x.u' * temp_u) * x.u';
    temp_u = temp_u + temp_u'; % temp_u = (u * u^T)'

    temp_v = x.d \ (x.u' * eta.TV);
    temp_v = x.v * temp_v - x.v * (temp_v * x.v) * x.v';
    temp_v = temp_v + temp_v'; % temp_v = (v * v^T)'
    
    temp_1 = temp_u * ((M_s * x.v) * x.v' - M_s);
    temp_2 = (x.u * (x.u' * M_s) - M_s) * temp_v;
    tempa = x.u * (x.u' * M_v);
    temp_3 = (tempa * x.v) * x.v' - tempa;
    temp_4 = (M_v * x.v) * x.v';
    output.TV = 2 * (temp_1 + temp_2 - temp_3 + temp_4);      
    output = project(x, output);      
end

function output = STV(x, a, eta)
    if(~isfield(eta, 'TV'))
        output.U = a * eta.U;
        output.D = a * eta.D;
        output.V = a * eta.V;
    else
        output.TV = a * eta.TV;
    end
end

function output = TVaddTV(x, eta1, eta2)
    if(~isfield(eta1, 'TV') || ~isfield(eta2, 'TV'))
        output.U = eta1.U + eta2.U;
        output.D = eta1.D + eta2.D;
        output.V = eta1.V + eta2.V;
    else
        output.TV = eta1.TV + eta2.TV;
    end
end

function output = Hv(x, H, v)
    [m, n] = size(v.v);
    v.v = reshape(v.v, m*n, 1);
    output.v = reshape(H * v.v, m, n);
end

function output = Hv_intr(x, H, v)
    output.v = H * v.v;
end

function output = rank1operator(x, v1, v2)
    [m, n] = size(v1.v);
    vv1 = reshape(v1.v, m * n, 1);
    vv2 = reshape(v2.v, m * n, 1);
    output = vv1 * vv2';
end

function output = rank1operator_intr(x, v1, v2)
    output = v1.v * v2.v';
end

function output = operadd(x, H1, H2)
    output = H1 + H2;
end

function [output, d, x, y] = TranspVect(x, d, y, v)

        ip_old = innerproduct(x, v, v);

        Av = x.v' * y.v;
        Au = x.u' * y.u;
    %     Bv = v.V' * y.v;
    %     Bu = v.U' * y.u;

        M1 = Au' * v.D * Av;
    %     U1 = x.u * (v.D * Bv);
    %     V1 = x.v * (v.D' * Au);

    %     M2 = Bu' * Av;
        U2 = v.U * Av;
    %     V2 = x.v * Bu;

    %     M3 = Au' * Bv;
    %     U3 = x.u * Bv;
        V3 = v.V * Au;

    %     output.D = M1 + M2 + M3;
    %     temp1 = U1 + U2 + U3;
    %     output.U = temp1 - y.u * (y.u' * temp1);
    %     temp2 = V1 + V2 + V3;
    %     output.V = temp2 - y.v * (y.v' * temp2);
        output.D = M1;
        output.U = U2 - y.u * (y.u' * U2);
        output.V = V3 - y.v * (y.v' * V3);

    ip_new = innerproduct(x, output, output);
    output = STV(x, ip_old / ip_new, output);

end

%%=======================assistant function==================%%
function [output, R] = qf(B)
    k = size(B, 2);
    [output, R] = qr(B, 0);
    diagR = diag(R);
    diagR(diagR == 0) = 1;
    output = output * spdiags(sign(diagR), 0, k, k);
end

function [output, R] = qf_thin(B)
    [output, R] = qr(B, 0);
    diagR = diag(R);
    output = output(:, abs(diagR) > 1e-10);
    output = output * spdiags(sign(diagR), 0, size(output, 2), size(output, 2));
end

function output = intr_c(x, eta)
   [m, k] = size(x.u);
   n = size(x.v, 1);
   if(~isfield(x, 'uperp'))
       x.uperp = spnull(x.u');
   end
   if(~isfield(x, 'vperp'))
       x.vperp = spnull(x.v');
   end
   if(~isfield(x, 'dim'))
        x.dim = (m + n - k) * k;
   end
   if(~isfield(x, 'ind_1'))
       x.ind_1 = find(tril(ones(k, k), -1));
   end
   if(~isfield(x, 'ind_2'))
       x.ind_2 = find(triu(ones(k,k), 1));
   end

    Omega = x.u' * eta.v * x.v;
    D_omega = diag(Omega);
    K1 = x.uperp' * eta.v * x.v;
    K2 = x.u' * eta.v * x.vperp;
    
    output.v = zeros(x.dim, 1);
    output.v(1 : 0.5*k*(k-1)) = Omega(x.ind_1);
    output.v(0.5*k*(k-1)+1 : 0.5*k*(k-1)+k) = D_omega;
    output.v(0.5*k*(k-1)+k+1 : k*k) = Omega(x.ind_2);
    output.v(k*k+1 : m*k) = reshape(K1, [], 1);
    output.v(m*k+1 : end) = reshape(K2, [], 1);
end

function output = full_c(x, xi)
   [m, k] = size(x.u);
   n = size(x.v, 1);
   if(~isfield(x, 'uperp'))
       x.uperp = spnull(x.u');
   end
   if(~isfield(x, 'vperp'))
       x.vperp = spnull(x.v');
   end
   if(~isfield(x, 'ind_1'))
       x.ind_1 = find(tril(ones(k, k), -1));
   end
   if(~isfield(x, 'ind_2'))
       x.ind_2 = find(triu(ones(k,k), 1));
   end
   ind_3 = find(eye(k));
   
    A = zeros(k,k);
    A(x.ind_1) = xi.v(1 : 0.5*k*(k - 1));
    A(ind_3) = xi.v(0.5*k*(k-1)+1 : 0.5*k*(k-1)+k); %temp;
    A(x.ind_2) = xi.v(0.5*k*(k-1)+k+1 : k*k);
   
    B = reshape(xi.v(k*k+1 : m*k), m-k, k);
    C = reshape(xi.v(m*k+1 : end), k, n-k);
    
    output.v = x.u * A * x.v' + x.uperp * B * x.v' + x.u * C * x.vperp' ;
end



