clear all

%Set the weighting matrix variable.
% which_Q = 'full_Q': the weighting matrix is a dense matrix
% which_Q = 'block_Q': the weighting matrix is a block diagonal matrix
which_Q = 'Projection_Q';
which_A = 'WC';    % 'WC' for well conditioned, 'IC' for ill conditioned   % 11PA
Nsample = 1;   % Sugg: 10   % 11PA

%rand('state', Nsample);  % deprecated
%randn('state', Nsample);  % deprecated
rng(0,'twister');  % 11PA - resets the random number generator

method_list = {
    'RRAM'
    'DMM'
    'SULS'
    'APM'
    };

% Dimensions, true rank and different max K:
%m = 80; n = 10; r_true = 5;
m = 50; n = 50; r_true = 2;  % 11PA
%K = [3, 5, 7];
K = [3];  % 11PA

for i = 1 : length(K)
    k = K(i);
    times1 = 0; times2 = 0; times3 = 0; times4 = 0;
    timecost1 = 0; timecost2 = 0; timecost3 = 0; timecost4 = 0;
    RErr1 = 0; RErr2 = 0; RErr3 = 0; RErr4 = 0;
    Err1 = 0; Err2 = 0; Err3 = 0; Err4 = 0;
    r1 = zeros(Nsample, 1); r2 = zeros(Nsample, 1);
    r3 = zeros(Nsample, 1); r4 = zeros(Nsample, 1);
    f1 = 0; f2 = 0; f3 = 0; f4 = 0;
    
    while (times1 < Nsample)
        
        %% generating data and weighting matrices
        switch which_A
            case 'WC'
                U_A = randn(m, r_true);
                V_A = randn(n, r_true);
                A = U_A * V_A';   % 11PA - previous version
                d = svd(A); 
                d(d>1.e-8)
            case 'IC'
                % 11PA - Generate ill-conditioned data matrix A:
                [U_A,~] = qr(randn(m,r_true),0);   % left singular vectors
                [V_A,~] = qr(randn(n,r_true),0);   % right singular vectors
                diagS_A = 10.^linspace(0,-10,r_true);   % singular values, logarithmically equally spaced
                A = U_A * diag(diagS_A) * V_A';   % matrix A
        end  % switch
        
        switch which_Q
            case 'full_Q'
                a = logspace(-2,0,m*n);
                S = a .* (0.5 + rand(1, m*n));
                [U, ~] = qr(randn(m*n), 0);   % 11PA - replaced rand by randn
                Q = U * diag(S) * U';
            case 'block_Q'
                a = logspace(-2,0,m);
                for j = 1 : n
                    S = a .* (0.5 + rand(1, m));
                    U = orth(rand(m));
                    Q0(:,:,j) = U * diag(S) * U';
                end
                Q = sparse(m*n, m*n);
                for j = 1:n
                    Q((j-1)*m+1:j*m, (j-1)*m+1:j*m) = Q0(:,:,j);
                end
                %                 Q = Q0(:, :, 1);
                %                 for j = 2 : size(Q0, 3)
                %                     Q = blkdiag(Q, Q0(:,:,j));
                %                 end
                %                 Q = sparse(Q);
            case 'Projection_Q'
                Q = sparse(m*n, m*n);
                OS = 2;
                dof = r_true*(m+n-r_true);
                samples = floor(OS * dof);
%                 samples = m*n;
                sample_ratio = samples/(m*n)
%                 samples = 0.3*r_true*(m+n);
                Omega = make_rand_Omega(m,n,samples);
                for j = 1:length(Omega)
                    Q(Omega(j),Omega(j))=1;
                end
        end
        
        %x0.u = orth(randn(m, 1)); x0.d = randn(1); x0.v = orth(randn(n, 1));
        x0.u = orth(randn(m, k)); x0.d = diag(rand(1,k)); x0.v = orth(randn(n, k));  % 11PA - note that d must be diagonal
        x0.A = A + 1.e-10*(2*rand(m,n)-1); [u_temp,d_temp,v_temp] = svd(x0.A); 
%         d_temp(d_temp>1.e-8)
%         norm(u_temp*u_temp' - U_A*U_A','fro')
        x0.u = u_temp(:,1:k); x0.d = diag(diag(d_temp(1:k,1:k))); x0.v = u_temp(:,1:k);
        
        %% MROM
        fprintf('*******************************MROM************************************** \n')
        %        [R1, err1, R_Err1, t1, fend1,G_MROM, F_MROM, T_MROM] = test_wlr(A, Q, k, x0, 2, 4, 10);
        %[R1, err1, R_Err1, t1, fend1,G_MROM, F_MROM, T_MROM] = test_wlr(A, Q, k, x0, 2, 4, 10);
        [R1, err1, R_Err1, t1, fend1,G_MROM, F_MROM, T_MROM] = test_wlr(A, Q, k, x0, 1, 4, 10);  % 11PA - method 7 is RSD.m, see MROM.m; retraction 1 is "SVD"
        timecost1 = timecost1 + t1; times1 = times1 + 1; f1 = f1 + fend1;
        Err1 = Err1 + err1; RErr1 = RErr1 + R_Err1; r1(times1) = rank(R1{end});
        
        %% Manton Newton
        fprintf('*******************************Manton Newton************************************** \n')
%                 [R2, t2, R_Err2, err2, fend2, F_Manton, T_Manton] = test_manton_weighted(A, Q, k, 11, 12);
%                 timecost2 = timecost2 + t2; times2 = times2 + 1; f2 = f2 + fend2;
%                 Err2 = Err2 + err2; RErr2 = RErr2 + R_Err2;
%                 a = svd(R2); r2(times2) = sum(a > 1e-08);
%                 sparse(a(a > 1e-08))
        
        %% SU Line-search
        fprintf('*******************************SU Line-Search************************************** \n')
                [R3, R_Err3, err3, t3, fend3, iter, F_SU, T_SU] = SU_linesearch(A, Q, k, x0);
                timecost3 = timecost3 + t3; times3 = times3 + 1; f3 = f3 + fend3;
                Err3 = Err3 + err3; RErr3 = RErr3 + R_Err3;
                a = svd(R3); r3(times3) = sum(a > 1e-08);
                sparse(a(a > 1e-08))
        
        %% Alternating Projection
        fprintf('*****************************Alternating Projection************************************** \n')
%                 [opt.p0, ~] = qr(randn(m, k), 0); opt.TolFun = 1e-07; opt.MaxIter = 2000;
%                 [p, info, R4, F_AP, G_AP, T_AP] = wtlsap(A, Q, k, opt);
%                 timecost4 = timecost4 + info.time; times4 = times4 + 1;
%                 temp = A - R4; fend4 = temp(:)' * Q * temp(:);
%                 f4 = f4 + fend4; RErr4 = RErr4 + sqrt(fend4 / (A(:)' * Q * A(:)));
%                 Err4 = Err4 + norm(temp, 'fro');
%                 a = svd(R4); r4(times4) = sum(a > 1e-08);
%                 sparse(a(a > 1e-08))
    end
    
    num_rank_MROM(i) = sum(r1)/times1; num_rank_DMM(i) = sum(r2)/times2;
    num_rank_SULS(i) = sum(r3)/times3; num_rank_AP(i) = sum(r4)/times4;
    
    feval_MROM(i) = f1/times1; feval_DMM(i) = f2/times2;
    feval_SULS(i) = f3/times3; feval_AP(i) = f4/times4;
    
    Rel_err_MROM(i) = RErr1/times1; Rel_err_DMM(i) = RErr2/times2;
    Rel_err_SULS(i) = RErr3/times3; Rel_err_AP(i) = RErr4/times4;
    
    Error_MROM(i) = Err1/times1; Error_DMM(i) = Err2/times2;
    Error_SULS(i) = Err3/times3; Error_AP(i) = Err4/times4;
    
    time_MROM(i) = timecost1/times1; time_DMM(i) = timecost2/times2;
    time_SULS(i) = timecost3/times3; time_AP(i) = timecost4/times4;
    
    if (k > r_true)
        count_r_MROM(i) = sum(r1 == r_true);count_r_DMM(i) = sum(r2 == r_true);
        count_r_SULS(i) = sum(r3 == r_true);count_r_AP(i) = sum(r4 == r_true);
    end
    
    
    % Figures:
    % Nick Trefethen's settings for fontsizes (obtained from Plamen Koev):
    %set(0,'defaultaxesfontsize',12, 'defaultaxeslinewidth',0.7, 'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
    
    % Mariya Ishteva uses also the following:
    %set(0,'defaultlinemarkersize',10)
    
    if (k >= r_true)
        figure(i);
        clf;   % 11PA
        semilogy(T_MROM, F_MROM, 'k-');
        hold on;
        semilogy(T_SU, F_SU, 'r--');
        hold on;
%         semilogy(T_Manton, F_Manton, 'r-.');
        hold on;
%         semilogy(T_AP, F_AP, 'b:');
        legend('RRAM', 'SULS')
        %legend('Algorithm 2', 'SULS', 'APM')
        xlabel('time'); ylabel('f')
        str=sprintf('k = %d', k);
        title(str)
        saveas(gcf,['k',num2str(k),'-diffmeth'],'fig');
        saveas(gcf,['k',num2str(k),'-diffmeth'],'png');
        saveas(gcf,['k',num2str(k),'-diffmeth'],'epsc');  % then use epstopdf to generate pdf
    end
    
    %     fprintf('MROM & %f & $%8.3e$ & $%8.3e$ & $%8.3e$ & $%8.3e$ \n', sum(r1)/times1, f1/times1, RErr1/times1, Err1/times1, timecost1/times1)
    %     fprintf('Algorithm 3 & %f & $%8.3e$ & $%8.3e$ & $%8.3e$ & $%8.3e$ \n', sum(r2)/times2, f2/times2, RErr2/times2, Err2/times2, timecost2/times2)
    %     fprintf('SU Line-search & %f & $%8.3e$ & $%8.3e$ & $%8.3e$ & $%8.3e$ \n', sum(r3)/times3, f3/times3, RErr3/times3, Err3/times3, timecost3/times3)
    %     fprintf('APM & %f & $%8.3e$ & $%8.3e$ & $%8.3e$ & $%8.3e$ \n', sum(r4)/times4, f4/times4, RErr4/times4, Err4/times4, timecost4/times4)
    
end


% Generating table:
% 11PA - Note that SULS and DMM are swapped: SULS is shown 2nd and DMM 3rd.
fid = fopen('MatrixCompletion.txt', 'w');
%fprintf(fid, '\\caption{Method comparison. The number in the parenthesis indicates the fraction of experiments where the numerical rank (number of singular values greater than $10^{-8}$). \n');
%fprintf(fid, '\\begin{table}[h] \n');
%fprintf(fid, '%s \n',strcat('\begin{tabular}{|c|',repmat('c|',1,6),'}'));
fprintf(fid, '%s \n','\begin{tabular}{cccccc}');
fprintf(fid,'%s \n','\hline');
fprintf(fid, 'k & method & rank & f & Relative Error & time(sec) \\\\ \n');
fprintf(fid,'%s \n','\hline');
for i = 1 : length(K)
    if(K(i) > r_true)
        fprintf(fid, 'k = %d & \\texttt{%s} & %.1f (%d/%d) & %s & %s & %s \\\\ \n', K(i), method_list{1},num_rank_MROM(i),count_r_MROM(i),Nsample, num2table(feval_MROM(i)), num2table(Rel_err_MROM(i)),num2table(time_MROM(i)));
        fprintf(fid, '       & \\texttt{%s} & %.1f (%d/%d) & %s & %s & %s \\\\ \n', method_list{3},num_rank_SULS(i),count_r_SULS(i),Nsample,num2table(feval_SULS(i)), num2table(Rel_err_SULS(i)),num2table(time_SULS(i)));
        fprintf(fid, '       & \\texttt{%s} & %.1f (%d/%d) & %s & %s & %s \\\\ \n', method_list{2},num_rank_DMM(i),count_r_DMM(i),Nsample,num2table(feval_DMM(i)), num2table(Rel_err_DMM(i)),num2table(time_DMM(i)));
        fprintf(fid, '       & \\texttt{%s} & %.1f (%d/%d) & %s & %s & %s \\\\ \n', method_list{4},num_rank_AP(i),count_r_AP(i),Nsample,num2table(feval_AP(i)), num2table(Rel_err_AP(i)),num2table(time_AP(i)));
    else
        fprintf(fid, 'k = %d & \\texttt{%s} & %.1f & %s & %s & %s \\\\ \n', K(i), method_list{1},num_rank_MROM(i),num2table(feval_MROM(i)), num2table(Rel_err_MROM(i)),num2table(time_MROM(i)));
        fprintf(fid, '       & \\texttt{%s} & %.1f & %s & %s & %s \\\\\n', method_list{3},num_rank_SULS(i),num2table(feval_SULS(i)), num2table(Rel_err_SULS(i)),num2table(time_SULS(i)));
        fprintf(fid, '       & \\texttt{%s} & %.1f & %s & %s & %s \\\\ \n', method_list{2},num_rank_DMM(i),num2table(feval_DMM(i)), num2table(Rel_err_DMM(i)),num2table(time_DMM(i)));
        fprintf(fid, '       & \\texttt{%s} & %.1f & %s & %s & %s \\\\\n', method_list{4},num_rank_AP(i),num2table(feval_AP(i)), num2table(Rel_err_AP(i)),num2table(time_AP(i)));
    end
    fprintf(fid, '%s \n', '\hline');
end
fprintf(fid,'%s \n','\end{tabular}');
fclose(fid);


% % Generating graphs:
% K = [5, 7];
% for i = 1 : length(K)
%     k = K(i);
%     A = randn(m, r_true) * randn(r_true, n);
%     switch which_Q
%         case 'full_Q'
%         	a = logspace(-2,0,m*n);
%           	S = a .* (0.5 + rand(1, m*n));
%           	[U, ~] = qr(rand(m*n), 0);
%          	Q = U * diag(S) * U';
%     	case 'block_Q'
%            	a = logspace(-2,0,m);
%            	for j = 1 : n
%               	S = a .* (0.5 + rand(1, m));
%               	U = orth(rand(m));
%                	Q0(:,:,j) = U * diag(S) * U';
%             end
%         	Q = sparse(m*n, m*n);
%            	for j = 1:n
%                 Q((j-1)*m+1:j*m, (j-1)*m+1:j*m) = Q0(:,:,j);
%             end
%     end
%
% 	%% MROM
%   	fprintf('*******************************MROM************************************** \n')
%   	x0.u = orth(randn(m, 1)); x0.d = randn(1); x0.v = orth(randn(n, 1));
%   	[R1, err1, R_Err1, t1, fend1,G_MROM, F_MROM, T_MROM] = test_wlr(A, Q, k, x0, 2, 4, 10);
%
%  	%% Manton Newton
%    	fprintf('*******************************Manton Newton************************************** \n')
%  	[R2, t2, R_Err2, err2, fend2, F_Manton, T_Manton] = test_manton_weighted(A, Q, k, 11, 12);
%
%  	%% SU Line-search
%  	fprintf('*******************************SU Line-Search************************************** \n')
%   	[R3, R_Err3, err3, t3, fend3, iter, F_SU, T_SU] = SU_linesearch(A, Q, k, x0);
%
%     %% Alternating Projection
%  	fprintf('*****************************Alternating Projection************************************** \n')
%    	[opt.p0, ~] = qr(randn(m, k), 0); opt.TolFun = 1e-07; opt.MaxIter = 2000;
%  	[p, info, R4, F_AP, G_AP, T_AP] = wtlsap(A, Q, k, opt);
%
%   	figure(i);
%     semilogy(T_MROM, F_MROM, 'k');
%     hold on;
%     semilogy(T_Manton, F_Manton, 'r');
%     hold on;
%     semilogy(T_SU, F_SU, 'g');
%     hold on;
%     semilogy(T_AP, F_AP, 'b');
%     legend('Algorithm 2', 'DMM', 'SULS', 'APM')
%     xlabel('cost time'); ylabel('f(x_i)')
%     str=sprintf('k = %d', k);
%     title(str)
%
% end



